## $Id: browser.py 17041 2022-07-27 14:54:11Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""UI components for basic applicants and related components.
"""
import grok
from zope.component import getUtility
from zope.i18n import translate
from waeup.kofa.interfaces import IExtFileStore, IKofaUtils
from waeup.kofa.widgets.datewidget import FriendlyDatetimeDisplayWidget
from zope.formlib.textwidgets import BytesDisplayWidget
from waeup.kofa.students.interfaces import IStudentsUtils
from waeup.kofa.applicants.browser import (ApplicantDisplayFormPage,
    ApplicantManageFormPage, ApplicantEditFormPage,
    ApplicantRegistrationPage,
    OnlinePaymentDisplayFormPage,
    OnlinePaymentBreadcrumb, ExportPDFPaymentSlipPage,
    AdditionalFile
    )
from waeup.kofa.applicants.viewlets import (
    PDFActionButton, PaymentReceiptActionButton)
from waeup.kofa.applicants.pdf import PDFApplicationSlip
from kofacustom.nigeria.applicants.browser import (
    NigeriaApplicantDisplayFormPage,
    NigeriaApplicantManageFormPage,
    NigeriaApplicantEditFormPage,
    NigeriaPDFApplicationSlip)
from kofacustom.nigeria.applicants.interfaces import (
    INigeriaPGApplicant, INigeriaUGApplicant,
    INigeriaPGApplicantEdit, INigeriaUGApplicantEdit,
    INigeriaApplicantOnlinePayment,
    #UG_OMIT_DISPLAY_FIELDS,
    #UG_OMIT_PDF_FIELDS,
    #UG_OMIT_MANAGE_FIELDS,
    #UG_OMIT_EDIT_FIELDS,
    PG_OMIT_DISPLAY_FIELDS,
    PG_OMIT_PDF_FIELDS,
    PG_OMIT_MANAGE_FIELDS,
    PG_OMIT_EDIT_FIELDS,
    )
from kofacustom.edocons.applicants.interfaces import (
    ICustomPGApplicant, ICustomUGApplicant, ICustomApplicant,
    ICustomPGApplicantEdit, ICustomUGApplicantEdit,
    ICustomApplicantOnlinePayment, ITranscriptApplicant,
    )
from kofacustom.nigeria.interfaces import MessageFactory as _

# Fields to be omitted in all display forms. course_admitted is
# rendered separately.

OMIT_DISPLAY_FIELDS = ('locked', 'course_admitted',
    'result_uploaded', 'suspended', 'special_application',
    'bank_account_number',
    'bank_account_name',
    'bank_name',
    #'course1', 'course2' # these 2 have been added and later removed again
    )

# UG students are all undergraduate students.
UG_OMIT_DISPLAY_FIELDS = OMIT_DISPLAY_FIELDS + (
    'jamb_subjects_list', 'programme_type')
UG_OMIT_PDF_FIELDS = UG_OMIT_DISPLAY_FIELDS + ('phone',)
UG_OMIT_MANAGE_FIELDS = (
    'special_application',
    'jamb_subjects_list',
    'programme_type',
    #'course1', 'course2', # these 2 have been added and later removed again
    )
UG_OMIT_EDIT_FIELDS = UG_OMIT_MANAGE_FIELDS + OMIT_DISPLAY_FIELDS + (
    'student_id',
    'notice',
    'screening_score',
    'screening_venue',
    'screening_date',
    'cbt_score',
    'cbt_venue',
    'cbt_date',
    'jamb_age',
    'jamb_subjects',
    'jamb_score',
    'jamb_reg_number',
    'aggregate')

TSC_OMIT_FIELDS = ('locked', 'suspended',
    )
    

TSC_OMIT_EDIT_FIELDS = TSC_OMIT_FIELDS + (
    'applicant_id',
    'proc_date',
    'courier_tno',
    )

TSC_OMIT_MANAGE_FIELDS = TSC_OMIT_FIELDS + (
    'applicant_id',)     

class CustomApplicantDisplayFormPage(NigeriaApplicantDisplayFormPage):
    """A display view for applicant data.
    """

    @property
    def form_fields(self):
        if self.target is not None and self.target.startswith('pg'):
            form_fields = grok.AutoFields(ICustomPGApplicant)
            for field in PG_OMIT_DISPLAY_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('tsc'):
            form_fields = grok.AutoFields(ITranscriptApplicant)
            for field in TSC_OMIT_FIELDS:
                form_fields = form_fields.omit(field)            
            return form_fields
        else:
            form_fields = grok.AutoFields(ICustomUGApplicant)
            for field in UG_OMIT_DISPLAY_FIELDS:
                form_fields = form_fields.omit(field)
            form_fields['notice'].custom_widget = BytesDisplayWidget
        #form_fields['perm_address'].custom_widget = BytesDisplayWidget
        if not getattr(self.context, 'student_id'):
            form_fields = form_fields.omit('student_id')
        if not getattr(self.context, 'screening_score'):
            form_fields = form_fields.omit('screening_score')
        if not getattr(self.context, 'screening_venue') or self._not_paid():
            form_fields = form_fields.omit('screening_venue')
        if not getattr(self.context, 'screening_date') or self._not_paid():
            form_fields = form_fields.omit('screening_date')
        if not getattr(self.context, 'cbt_score'):
            form_fields = form_fields.omit('cbt_score')
        if not getattr(self.context, 'cbt_venue') or self._not_paid():
            form_fields = form_fields.omit('cbt_venue')
        if not getattr(self.context, 'cbt_date') or self._not_paid():
            form_fields = form_fields.omit('cbt_date')
        return form_fields

class CustomPDFApplicationSlip(NigeriaPDFApplicationSlip):

    @property
    def form_fields(self):
        if self.target is not None and self.target.startswith('pg'):
            form_fields = grok.AutoFields(ICustomPGApplicant)
            for field in PG_OMIT_PDF_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('tsc'):
            form_fields = grok.AutoFields(ITranscriptApplicant)
            for field in TSC_OMIT_FIELDS:
                form_fields = form_fields.omit(field)              
            return form_fields
        else:
            form_fields = grok.AutoFields(ICustomUGApplicant)
            for field in UG_OMIT_PDF_FIELDS:
                form_fields = form_fields.omit(field)
        if not getattr(self.context, 'student_id'):
            form_fields = form_fields.omit('student_id')
        if not getattr(self.context, 'screening_score'):
            form_fields = form_fields.omit('screening_score')
        if not getattr(self.context, 'screening_venue'):
            form_fields = form_fields.omit('screening_venue')
        if not getattr(self.context, 'screening_date'):
            form_fields = form_fields.omit('screening_date')
        if not getattr(self.context, 'cbt_score'):
            form_fields = form_fields.omit('cbt_score')
        if not getattr(self.context, 'cbt_venue'):
            form_fields = form_fields.omit('cbt_venue')
        if not getattr(self.context, 'cbt_date'):
            form_fields = form_fields.omit('cbt_date')
        return form_fields

class CustomApplicantManageFormPage(NigeriaApplicantManageFormPage):
    """A full edit view for applicant data.
    """

    def display_fileupload(self, filename):
        if filename[1] == 'res_stat' \
            and self.target is not None \
            and not self.target.startswith('tsc'):
            return False
        if filename[1] == 'eligibility' \
            and self.target is not None \
            and not self.target.startswith('tsc'):
            return False
        return True

    @property
    def form_fields(self):
        if self.target is not None and self.target.startswith('pg'):
            form_fields = grok.AutoFields(ICustomPGApplicant)
            for field in PG_OMIT_MANAGE_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('tsc'):
            form_fields = grok.AutoFields(ITranscriptApplicant)
            for field in TSC_OMIT_MANAGE_FIELDS:
                form_fields = form_fields.omit(field)              
            return form_fields
        else:
            form_fields = grok.AutoFields(ICustomUGApplicant)
            for field in UG_OMIT_MANAGE_FIELDS:
                form_fields = form_fields.omit(field)
        form_fields['student_id'].for_display = True
        form_fields['applicant_id'].for_display = True
        return form_fields

class CustomApplicantEditFormPage(NigeriaApplicantEditFormPage):
    """An applicant-centered edit view for applicant data.
    """

    def unremovable(self, ticket):
        return True

    def display_fileupload(self, filename):
        if filename[1] == 'res_stat' \
            and self.target is not None \
            and not self.target.startswith('tsc'):
            return False
        if filename[1] == 'eligibility' \
            and self.target is not None \
            and not self.target.startswith('tsc'):
            return False
        return True

    @property
    def form_fields(self):
        if self.target is not None and self.target.startswith('pg'):
            form_fields = grok.AutoFields(ICustomPGApplicantEdit)
            for field in PG_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)
        elif self.target is not None and self.target.startswith('tsc'):
            form_fields = grok.AutoFields(ITranscriptApplicant)
            for field in TSC_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)            
            return form_fields
        else:
            form_fields = grok.AutoFields(ICustomUGApplicantEdit)
            for field in UG_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)
        form_fields['applicant_id'].for_display = True
        form_fields['reg_number'].for_display = True
        return form_fields

    def dataNotComplete(self, data):
        store = getUtility(IExtFileStore)
        if self.context.__parent__.with_picture \
            and self.context.__parent__.picture_editable:
            store = getUtility(IExtFileStore)
            if not store.getFileByContext(self.context, attr=u'passport.jpg'):
                return _('No passport picture uploaded.')
        if self.target is not None \
            and self.target.startswith('tscf') \
            and not store.getFileByContext(self.context, attr=u'res_stat.pdf'):
            return _('No statement of result pdf file uploaded.')
        #if self.target is not None \
        #    and self.target.startswith('tsc') \
        #    and not store.getFileByContext(self.context, attr=u'eligibility.pdf'):
        #    return _('No eligibility form pdf file uploaded.')
        return False

class ResultStatement(AdditionalFile):
    grok.name('res_stat')

class EligibilityForm(AdditionalFile):
    grok.name('eligibility')        